'use strict';

var fs = require('fs-extra');
var libFsExtra = require('fs-extra');
var exec = require('child_process').exec;
var execSync = require('child_process').execSync;
var libQ = require('kew');
const { setFlagsFromString } = require('v8');
var config = new (require('v-conf'))();

// Define the peppyspectrum class
module.exports = peppyspectrum;

function peppyspectrum(context) {
    const self = this;
    self.context = context;
    self.commandRouter = self.context.coreCommand;
    self.logger = self.commandRouter.logger;
    this.context = context;
    this.commandRouter = this.context.coreCommand;
    this.logger = this.context.logger;
    this.configManager = this.context.configManager;
};

peppyspectrum.prototype.onVolumioStart = function () {
    const self = this;
    var configFile = self.commandRouter.pluginManager.getConfigurationFile(self.context, 'config.json');
    self.config = new (require('v-conf'))();
    self.config.loadFile(configFile);
    return libQ.resolve();
};

peppyspectrum.prototype.getConfigurationFiles = function () {
    return ['config.json'];
};

// Plugin methods -----------------------------------------------------------------------------

peppyspectrum.prototype.onStop = function () {
    const self = this;
    let defer = libQ.defer();
    self.logger.info("Stopping peppyspectrum service");
    self.commandRouter.stateMachine.stop().then(function () {
        exec("/usr/bin/sudo /bin/systemctl stop peppyspectrum.service", {
            uid: 1000,
            gid: 1000
        }, function (error, stdout, stderr) { })
        socket.off();
    });
    defer.resolve();
    return libQ.resolve();
};

peppyspectrum.prototype.onStart = function () {
    var self = this;
    var defer = libQ.defer();

     self.modprobedummy()
    self.commandRouter.executeOnPlugin('audio_interface', 'alsa_controller', 'updateALSAConfigFile')
  
      .then(function (e) {
        var pipeDefer = libQ.defer();
        exec("/usr/bin/mkfifo /tmp/peppy_meter_spectrum" + "; /bin/chmod 777 /tmp/peppy_meter_spectrum", { uid: 1000, gid: 1000 }, function (error, stdout, stderr) {
          if (error) {
            self.logger.warn("An error occurred when creating myfifosapeppy", error);
          }
          pipeDefer.resolve();
        });
        
      return Defer.promise;
    });
    defer.resolve();
    setTimeout(function () {
        self.startpeppyservice()
    }, 2000);
    self.commandRouter.pushToastMessage('success', 'Starting peppyspectrum');

    return defer.promise;
};


//here we load snd-dummy module
peppyspectrum.prototype.modprobedummy = function () {
    const self = this;
    let defer = libQ.defer();
    //self.hwinfo();
    try {
        execSync("/usr/bin/sudo /sbin/modprobe snd-dummy index=6 pcm_substreams=1", {
            uid: 1000,
            gid: 1000
        });
        self.commandRouter.pushConsoleMessage('snd-dummy loaded');
        defer.resolve();
    } catch (err) {
        self.logger.info('failed to load snd-dummy' + err);
    }
};


peppyspectrum.prototype.startpeppyservice = function () {
    const self = this;
    let defer = libQ.defer();

    exec("/usr/bin/sudo /bin/systemctl start peppyspectrum.service", {
        uid: 1000,
        gid: 1000
    }, function (error, stdout, stderr) {
        if (error) {
            self.logger.info('peppyspectrum failed to start. Check your configuration ' + error);
        } else {
            self.commandRouter.pushConsoleMessage('peppyspectrum Daemon Started');

            defer.resolve();
        }
    });
};

peppyspectrum.prototype.restartpeppyservice = function () {
    const self = this;
    let defer = libQ.defer();
    exec("/usr/bin/sudo /bin/systemctl restart peppyspectrum.service", {
        uid: 1000,
        gid: 1000
    }, function (error, stdout, stderr) {
        if (error) {
            self.logger.info('peppyspectrum failed to start. Check your configuration ' + error);
        } else {
            self.commandRouter.pushConsoleMessage('peppyspectrum Daemon Started');

            defer.resolve();
        }
    });
};

peppyspectrum.prototype.onRestart = function () {
    const self = this;
};

peppyspectrum.prototype.onInstall = function () {
    const self = this;
    //	//Perform your installation tasks here
};

peppyspectrum.prototype.onUninstall = function () {
    const self = this;
    //Perform your installation tasks here
};

peppyspectrum.prototype.getUIConfig = function () {
    const self = this;
    const defer = libQ.defer();
    var lang_code = this.commandRouter.sharedVars.get('language_code');
    self.commandRouter.i18nJson(__dirname + '/i18n/strings_' + lang_code + '.json',
        __dirname + '/i18n/strings_en.json',
        __dirname + '/UIConfig.json')
        .then(function (uiconf) {
            let spectrumfolder;
            var showsize = self.config.get("showsize")
            var autosize = self.config.get("auutosize")

            var valuescreen = self.config.get('screensize');
            self.configManager.setUIConfigParam(uiconf, 'sections[0].content[0].value.value', valuescreen);
            self.configManager.setUIConfigParam(uiconf, 'sections[0].content[0].value.label', valuescreen);


            const directoryPath = '/data/INTERNAL/PeppySpectrum/Templates/';

            // Use a Promise for asynchronous operations
            function readDirectory() {
                return new Promise((resolve, reject) => {
                    fs.readdir(directoryPath, (err, files) => {
                        if (err) {
                            reject(err);
                            return;
                        }

                        const folders = files.filter(file => fs.statSync(`${directoryPath}/${file}`).isDirectory());
                        resolve(folders);
                    });
                });
            }

            // Call the function
            readDirectory()
                .then(folders => {
                    console.log('Folders in the directory:', folders);

                    let allfolder = 'small,medium,large,wide,' + folders;
                    //   self.logger.info('list is ' + allfilter)
                    var litems = allfolder.split(',');

                    for (let a in litems) {
                        console.log('Text between brackets:', litems[a]);

                        self.configManager.pushUIConfigParam(uiconf, 'sections[0].content[0].options', {
                            value: litems[a],
                            label: litems[a]
                        });

                    }

                })


                .catch(err => console.error('Error reading directory:', err));
            /*   if (((valuescreen != 'small') || (valuescreen != 'medium') || (valuescreen != 'large'))) {
                   uiconf.sections[1].content[1].hidden = false;
                   uiconf.sections[1].content[2].hidden = false;
                   uiconf.sections[1].content[3].hidden = false;
   
               } else if ((showsize) && (autosize)){
                   uiconf.sections[1].content[1].hidden = false;
                   uiconf.sections[1].content[2].hidden = false;
               //    uiconf.sections[1].content[3].hidden = false;
   
               }else {
                   */
            uiconf.sections[1].content[1].hidden = true;
            uiconf.sections[1].content[2].hidden = true;
            uiconf.sections[1].content[3].hidden = true;

            //}

            var screenwidth = self.config.get('screenwidth')
            uiconf.sections[1].content[1].value = screenwidth;
            uiconf.sections[1].content[1].attributes = [
                {
                    placeholder: screenwidth,
                    min: 0,
                    max: 3500
                }
            ];


            var screenheight = self.config.get('screenheight')
            uiconf.sections[1].content[2].value = screenheight;
            uiconf.sections[1].content[2].attributes = [
                {
                    placeholder: screenheight,
                    min: 0,
                    max: 3500
                }
            ];


            var valuespectrum;
            valuespectrum = self.config.get('spectrum');
            self.configManager.setUIConfigParam(uiconf, 'sections[1].content[0].value.value', valuespectrum);
            self.configManager.setUIConfigParam(uiconf, 'sections[1].content[0].value.label', valuespectrum);

            try {
                if ((valuescreen == 'small') || (valuescreen == 'medium') || (valuescreen == 'large') || (valuescreen == 'wide')) {
                    spectrumfolder = '/data/plugins/user_interface/peppyspectrum/PeppySpectrum/'
                } else {
                    spectrumfolder = '/data/INTERNAL/PeppySpectrum/Templates/'
                }

                fs.readFile(spectrumfolder + valuescreen + '/spectrum.txt', function (err, idata) {
                    if (err) {
                        console.error('Error reading the file:', err);
                        self.configManager.pushUIConfigParam(uiconf, 'sections[1].content[0].options', {
                            value: 'no config!',
                            label: 'no config!'
                        });
                        //   return defer.reject(err); // Reject the promise in case of an error
                    }

                    const regex = /\[(.*?)\]/g;
                    let match;
                    const matches = [];
                    while ((match = regex.exec(idata)) !== null) {
                        matches.push(match[1]);
                    }
                    let allfilter = 'Random,' + matches;
                    self.logger.info('list is ' + allfilter)
                    var litems = allfilter.split(',');

                    for (let a in litems) {
                        console.log('Text between brackets:', litems[a]);

                        self.configManager.pushUIConfigParam(uiconf, 'sections[1].content[0].options', {
                            value: litems[a],
                            label: litems[a]
                        });
                    }
                    uiconf.sections[2].content[0].value = self.config.get('debuglog');

                    // Resolve the promise after the file reading and processing are complete
                    defer.resolve(uiconf);
                });
            } catch (e) {
                self.logger.error('Cannot read file: ' + e);
                defer.reject(e); // Reject the promise in case of an error
            }
        })
        .fail(function () {
            defer.reject(new Error());
        });
    return defer.promise;
};


peppyspectrum.prototype.getAdditionalConf = function (type, controller, data) {
    const self = this;
    return self.commandRouter.executeOnPlugin(type, controller, 'getConfigParam', data);
}

peppyspectrum.prototype.refreshUI = function () {
    const self = this;

    setTimeout(function () {
        var respconfig = self.commandRouter.getUIConfigOnPlugin('user_interface', 'peppyspectrum', {});
        respconfig.then(function (config) {
            self.commandRouter.broadcastMessage('pushUiConfig', config);
        });
        self.commandRouter.closeModals();
    }, 100);
}

peppyspectrum.prototype.getLabelForSelect = function (options, key) {
    var n = options.length;
    for (var i = 0; i < n; i++) {
        if (options[i].value == key)
            return options[i].label;
    }
    return 'VALUE NOT FOUND BETWEEN SELECT OPTIONS!';
};

peppyspectrum.prototype.savepeppy = function (data) {
    const self = this;

    const defer = libQ.defer();

    var screensize = (data['screensize'].value);
    // if (screensize == !('custom')) {
    //   var screenwidth = ''
    // var screenheight = ''
    //} else {

    // }
    self.config.set('screensize', screensize);
    var autovalue = screensize.split('_')

    if ((screensize == 'small') || (screensize == 'medium') || (screensize == 'large') || (screensize == 'wide')) {
        self.config.set('showsize', false);

    } else {
        self.config.set('showsize', true);
    }
    self.config.set('spectrum', 'Random');



    self.savepeppyconfig();
    self.restartpeppyservice()
    self.refreshUI()
        .then(function (e) {
            self.commandRouter.pushToastMessage('success', "peppyspectrum Configuration updated");
            defer.resolve({});
        })
        .fail(function (e) {
            defer.reject(new Error('error'));
            self.commandRouter.pushToastMessage('error', "failed to start. Check your config !");
        })
    return defer.promise;

};


peppyspectrum.prototype.savepeppy1 = function (data) {
    const self = this;

    const defer = libQ.defer();
    var screenwidth// = data["screenwidth"]
    var screenheight// = data["screenheight"]
    var screensize = self.config.get('screensize')
    //  var autovalue = screensize.split('_')
    var autovalue = screensize.split('(')
    if (Array.isArray(autovalue) && autovalue.length === 2) {
        console.log("Autovalue:", autovalue);
    } else {
        if ((screensize !== 'small' && screensize !== 'medium' && screensize !== 'large' && screensize !== 'wide') ) {
       
        console.error("Error splitting screensize:");

        console.log('The variable is not a finite number.');
        self.commandRouter.pushToastMessage('error', "Can't determine screen size because the folder is not properly named! Must be Mycustom(widthxheight) ex : Metal(800x600) please check!");

        screenwidth = '480';
        screenheight = '240';

        self.config.set('screenwidth', screenwidth);
        self.config.set('screenheight', screenheight);
        return;
        }
    }

    var sizef = autovalue[1]
    var size = sizef.slice(0, -1)
    //console.log('aaaaaaaaaaa '+size)


    var sizen = size.split('x')
    screenwidth = sizen[0]
    screenheight = sizen[1]
    let myNumberx = parseInt(screenwidth, 10);
    let myNumbery = parseInt(screenheight, 10);
    var truex = (typeof myNumberx === 'number' && isFinite(myNumberx))
    var truey = (typeof myNumbery === 'number' && isFinite(myNumbery))
    if (truex && truey) {
        console.log('The variable is a finite number.'+myNumberx+" "+myNumbery);
    } else if (((!truex && !truey)) || (size == undefined)) {
        console.log('The variable is not a finite number.');
        self.commandRouter.pushToastMessage('error', "Can't determine screen size because the folder is not properly named!,Must be Mycustom(widthxheight) ex : Metal(800x600), please check!");

        myNumberx = '480';
        myNumbery = '240';
    }

    self.config.set('spectrum', data['spectrum'].value);

    self.config.set('autosize', data['autosize']);

    self.config.set('screenwidth', myNumberx);
    self.config.set('screenheight', myNumbery);
    self.savepeppyconfig();
    self.restartpeppyservice()
        .then(function (e) {
            self.commandRouter.pushToastMessage('success', "peppyspectrum Configuration updated");
            defer.resolve({});
        })
        .fail(function (e) {
            defer.reject(new Error('error'));
            self.commandRouter.pushToastMessage('error', "failed to start. Check your config !");
        })
    return defer.promise;

};


peppyspectrum.prototype.savepeppy2 = function (data) {
    const self = this;

    const defer = libQ.defer();

    self.config.set('debuglog', data['debuglog']);

    self.savepeppyconfig();
    self.restartpeppyservice()
        .then(function (e) {
            self.commandRouter.pushToastMessage('success', "peppyspectrum Configuration for debug log updated");
            defer.resolve({});
        })
        .fail(function (e) {
            defer.reject(new Error('error'));
            self.commandRouter.pushToastMessage('error', "failed to start. Check your config !");
        })
    return defer.promise;

};


//here we save the asound.conf file config
peppyspectrum.prototype.savepeppyconfig = function () {
    const self = this;

    const defer = libQ.defer();
    try {

        fs.readFile(__dirname + "/config.txt.tmpl", 'utf8', function (err, data) {
            if (err) {
                defer.reject(new Error(err));
                return console.log(err);
            }
            var autosize = self.config.get('autosize')

            var screensize = self.config.get('screensize')
            if ((screensize == 'small') || (screensize == 'medium') || (screensize == 'large') || (screensize == 'wide')) {
                var screenwidth = ''
                var screenheight = ''

            } else {
                screensize = ('/data/INTERNAL/PeppySpectrum/Templates/' + self.config.get("screensize"))
                screenwidth = self.config.get("screenwidth")
                screenheight = self.config.get("screenheight")
            }


            var spectrum = self.config.get('spectrum')
            if (spectrum == 'Random') {
                spectrum = ''
            }

            var debuglog = self.config.get('debuglog')
            if (debuglog) {
                var debuglogd = 'True'
            }
            else if (debuglogd = 'False');

            const conf1 = data.replace("${spectrum}", spectrum)
                .replace("${screensize}", screensize)
                .replace("${screenwidth}", screenwidth)
                .replace("${screenheight}", screenheight)
                .replace("${debuglog}", debuglogd)


            fs.writeFile("/data/plugins/user_interface/peppyspectrum/PeppySpectrum/config.txt", conf1, 'utf8', function (err) {
                if (err)
                    defer.reject(new Error(err));
                else defer.resolve();
            });

        });
        self.refreshUI()

    } catch (err) {

    }
    return defer.promise;
};


peppyspectrum.prototype.setUIConfig = function (data) {
    const self = this;

};

peppyspectrum.prototype.getConf = function (varName) {
    const self = this;
    //Perform your installation tasks here
};


peppyspectrum.prototype.setConf = function (varName, varValue) {
    const self = this;
};
